#!/usr/bin/python
# -*- coding: utf-8 -*-

#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from qttable import *
from Driza.ui.ventanas.uivprincipal import vprincipal
from Driza.dialogs.dconfig import DConfig
from Driza.dialogs.dseleccionvar import DFiltro
from Driza.dialogs.dseleccionvar import DAgrupacion
from Driza.dialogs.dcrevar import DCrevar
from Driza.dialogs.dbuscar import DBuscar
from Driza.dialogs.dcasos import DCasos
from Driza.dialogs.doperaciones import DOperaciones
from Driza.dialogs.dimportartexto import DImportarTexto

from Driza import listas
import string
from Driza.widgets.widgetsqt import GrisTableItem

class VPrincipal(vprincipal):
	"""Ventana principal, hereda de vprincipal, que esta generado por el designer. Esta clase es padre de la mayoria de los dialogos, y es la que permite introducir los datos a el usuario"""

	def __init__(self,parent,datos,gestoroperaciones,gestorfiltro,config,portero,gestorficheros,gestortemas,gestoragrupacion): 
		"""Inicializador de la ventana principal"""
		vprincipal.__init__(self,None)
		#VARIABLES PUBLICAS
		self.__datos=datos
		self.parent=parent
		self.__config=config
		self.__gestortemas=gestortemas
		self.dcasos = DCasos(self,datos)#UElementos de la tabla de variables querran acceder
		#Bloque de inicialización de dialogos
#		self.dcalculo = DCalculo(gestoroperaciones,self,"Dialogo de calculo",0,)
#		self.__dgraficos = DGraficos(gestoroperaciones,self)
		self.__doperaciones = DOperaciones(self,gestoroperaciones,self.parent.vsalida)
		self.__dcrevar = DCrevar(self,datos,config)
		self.__dfiltro = DFiltro(gestorfiltro,datos,self.__dcrevar,self)
		self.__dagrupacion = DAgrupacion(gestoragrupacion,datos,self.__dcrevar,self)
		self.__dbuscar = DBuscar(self,"busqueda",0,)
		self.__dconfig = DConfig(config,self)
		self.__dimportartexto=DImportarTexto(self,gestorficheros)
		#Variables enteras que almacenan el numero de casillas deseadas para las tablas como excedente
		self.__nreg=10 
		self.__nvar=10
		self.__gestorfiltro=gestorfiltro
		self.__portero=portero
		self.__gestorfichero=gestorficheros


		# Configuracion de las tablas
		self.__inittabladat()
		self.__inittablavar(self.__nvar)
		# Configuracion de la toolbar
		self.__inittoolbar()
		self.__conexiones()

	#FUNCIONES PUBLICAS
	
	def __importar(self):
		"""Importa los datos de un fichero"""
		self.__dimportartexto.show()

	def abrirfichero(self):
		""" Lanza el diálogo de apertura de fichero """
		filter=""
		for fmt in listas.extensiones_fichero():
			filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
		fn = QFileDialog.getOpenFileName(QString.null,filter,self,None,"Dialogo abrir fichero","")
		filename = str(fn)
		if filename:
			self.__gestorfichero.cargar(filename)
		self.show()

	def show(self):
		"""Esta funcion actualiza el contenido de las rejillas segun los datos
		Reimplementa el show de la clase base
		"""
		#Tabla de datos
		#El margen extra sugerido por las variables nvar y ndat
		self.__showtablavar()
		self.__showtabladat()
		self.__showundoredo()
		if self.__gestorfichero.fichero:
			nfichero= self.__gestorfichero.fichero
		else:
			nfichero= "Nuevo fichero"
		self.setCaption("Driza "+self.parent.version()+" - "+nfichero)

		#Titulo de datos
		vprincipal.show(self)

	def closeEvent( self, ev ):
		"""Acciones que se realizaran cuando el usuario cierre la ventana. Guarda el estado de la configuración, incluyendo los ficheros recientemente abiertos"""
		from Driza.interfaz import InterfazConfig
		ic=InterfazConfig()
		ic.save(self.__config)
		QWidget.closeEvent( self, ev )

	#FUNCIONES PRIVADAS
	
	def __showundoredo(self):
		if self.__portero.puedoundo():
			self.editUndoAction.setEnabled(True)
		else:
			self.editUndoAction.setEnabled(False)

		if self.__portero.puedoredo():
			self.editRedoAction.setEnabled(True)
		else:
			self.editRedoAction.setEnabled(False)


	def __dacerca_de(self):
		"""Mensaje de about"""
		QMessageBox.about(self,"Acerca de Driza", u"Driza %s es una interfaz estadística\n (C) Néstor Arocha Rodríguez - Inmaculada luengo Merino\n Distribuido bajo licencia GPL" % (self.parent.version()))
	
	def __undo(self):
		"""Deshace el último cambio"""
		self.__portero.undo()
		self.show()
	
	def __redo(self):
		"""Rehace el último cambio"""
		self.__portero.redo()
		self.show()

	def __inittoolbar(self):
		"""Inicializa la toolbar"""
		self.boton1=QToolButton(self.Toolbar)
		self.boton1.setTextLabel("ABRIR")
	 	iconos=QIconSet()
		iconos.setIconSize(QIconSet.Small,QSize(100,100))
		iconos.setPixmap(self.__gestortemas.iconoabrir(),QIconSet.Small)
#		iconos.setPixmap("images/flechaabajo.png",QIconSet.Automatic,QIconSet.Normal,QIconSet.Off)
		self.boton1.setOn(True)
		self.boton1.setIconSet(iconos)
		self.boton2=QToolButton(self.Toolbar)
		self.boton2.setTextLabel("GUARDAR")
	 	iconos=QIconSet()
		iconos.setIconSize(QIconSet.Small,QSize(100,100))
		iconos.setPixmap(self.__gestortemas.iconoguardar(),QIconSet.Small)
		self.boton2.setOn(True)
		self.boton2.setIconSet(iconos)

	def __conexiones(self):
		"""Funcion llamada en el constructor que almacena todas las conexiones accion-funcion"""
		self.connect(self.fileExitAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.archivoSalirAction,SIGNAL("activated()"),self.__salirprograma)
		self.connect(self.fileNewAction,SIGNAL("activated()"),self.__nuevo)
		self.connect(self.fileOpenAction,SIGNAL("activated()"),self.abrirfichero)
		self.connect(self.fileSaveAsAction,SIGNAL("activated()"),self.__guardarcomo)
		self.connect(self.fileSaveAction,SIGNAL("activated()"),self.__guardar)
		self.connect(self.archivoImportarAction,SIGNAL("activated()"),self.__importar)
		self.connect(self.analizarMediaAction,SIGNAL("activated()"),self.__dcalculodescriptivo)
		self.connect(self.analizarBondad_de_ajusteJi_cuadradoAction,SIGNAL("activated()"),self.__dcalculojicuadrado)
		self.connect(self.AnalizarKs,SIGNAL("activated()"),self.__dcalculoks)
		self.connect(self.analizarRegresin_linealAction,SIGNAL("activated()"),self.__dcalculoregresion)
		self.connect(self.programaConfigurarAction,SIGNAL("activated()"),self.__dconfig.show)
		self.connect(self.table1,SIGNAL("valueChanged(int,int)"),self.__actualizardat)
		self.connect(self.table2,SIGNAL("valueChanged(int,int)"),self.__actualizarvar)
		self.connect(self.datosFiltrarAction,SIGNAL("activated()"),self.__dfiltro.show)
		self.connect(self.datosAgruparAction,SIGNAL("activated()"),self.__dagrupacion.show)
		self.connect(self.modificarCrear_nuevas_variablesAction,SIGNAL("activated()"),self.__dcrevar.show)
		self.connect(self.graficosHistogramaAction,SIGNAL("activated()"),self.__dgraficoshistograma)
		self.connect(self.DiagramaDispersionAction,SIGNAL("activated()"),self.__dgraficosdispersion)
		self.connect(self.distribucionesDistribucion_NormalAction,SIGNAL("activated()"),self.__ddistribucionnormal)
		self.connect(self.editFindAction,SIGNAL("activated()"),self.__dbuscar.show)
		self.connect(self.editCopyAction,SIGNAL("activated()"),self.__copy)
		self.connect(self.editCutAction,SIGNAL("activated()"),self.__cut)
		self.connect(self.editPasteAction,SIGNAL("activated()"),self.__paste)
		self.connect(self.editUndoAction,SIGNAL("activated()"),self.__undo)
		self.connect(self.editRedoAction,SIGNAL("activated()"),self.__redo)
		self.connect(self.acerca_deAcerca_deAction,SIGNAL("activated()"),self.__dacerca_de)
		self.connect(self.boton1,SIGNAL("clicked()"),self.abrirfichero)
		self.connect(self.boton2,SIGNAL("clicked()"),self.__guardar)

	def __copy(self):
		"""Funcion que copia y borra la seleccion"""
		self.__copiar()
		self.__removeselection(self.table1)

	
	def __cut(self):
		"""Copia, borra la seleccion y su contenido"""
		self.__portero.guardarestado()
		self.__copiar()
		self.__deleteselection(self.table1)
		self.show()


	def __paste(self):
		"""Pega lo que haya sido copiado"""
		self.__portero.guardarestado()
		cb = QApplication.clipboard()
		registro=cb.text().latin1().split(";")
		pos=self.table1.currentRow()
		del registro[-1]
		registro.reverse()
		for x in registro:
			elemento=x.split(":")
			del elemento[-1] #El ultimo elemento vacio
			lista=[]
			for valor in elemento:
				lista.append(valor)
			self.__datos.insReg(pos,lista)
			self.show()


	def __inittabladat(self):
		"""Inicializa la tabla de datos"""
		self.table1.setSelectionMode(QTable.MultiRow)
		self.table1.setNumCols(16) 
		self.table1.setNumRows(64)

	def __inittablavar(self,nvars):
		"""Inicializa la tabla de variables"""
		self.table2.setSelectionMode(QTable.MultiRow)
		self.table2.setNumRows(nvars)
		titulos=self.table2.horizontalHeader()
		titulos.setLabel (0,"Nombre")
		titulos.setLabel (1,"Tipo")
		titulos.setLabel (2,"Valores")
		titulos.setLabel (3,"Valor por defecto")
		titulos.setLabel (4,u"Descripción")
		self.table2.setNumCols(5)

	def __showtablavar(self,pos=None):  
		""" Representa la variable pos en la tabla de variables. Las propiedades de la variable las lee de los datos.
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:
			self.__showvar(pos)
			#Actualizacion de titulo en la tabla de datos
			self.__showtitulodat(pos)
		else:
			for n in range(self.__datos.numVar()):
				self.__showvar(n)
			for n in range(self.__datos.numVar(),self.table2.numRows()):
				for y in range(self.table2.numCols()):
					self.table2.clearCell(n,y)
			self.__showtitulodat()

	def __showvar(self,pos):
		"""Muestra una unica variable (fila) en la tabla segun el listado de variables"""
		variable=self.__datos.getVar(pos)
		self.table2.setItem(pos,2,self.__buttontableitem())
		self.table2.setText(pos,0,str(variable.nombre))
		self.table2.setText(pos,3,str(variable.valorpordefecto))
		self.table2.setText(pos,4,str(variable.descripcion))
		self.table2.setItem(pos,1,self.__combotableitem())	
		self.table2.item(pos,1).setCurrentItem(variable.tipo)

	def __showtabladat(self,pos=None):
		"""Muestra el dato en la posicion pos (fila)
		Lee de los datos, así que deben existir previamente
		Comprueba si esta filtrado o no para cambiar el color
		Si no se indica posicion, se entiende que se quiere rellenar toda la tabla
		"""
		if pos:  
			#Se ha indicado la posición
			self.__showdat(pos)
		else: 
			#No se ha indicado la posicion, se rellena todo
			for x in range(self.__datos.numReg()):
				self.__showdat(x)
			self.__showtitulodat()
			for x in range(self.__datos.numReg(),self.table1.numRows()):
				for y in range(self.table1.numCols()):
					self.table1.clearCell(x,y)
			for x in range(self.table1.numRows()):
				for y in range(self.__datos.numVar(),self.table1.numCols()):
					self.table1.clearCell(x,y)

	def __showdat(self,pos):
		"""Muestra un solo dato"""
		for i in range(self.__datos.numVar()):
			if self.__gestorfiltro.variable:
				if self.__datos.getCol(self.__gestorfiltro.variable)[pos]:
					self.table1.setItem(pos,i,QTableItem(self.table1,QTableItem.OnTyping,repr(self.__datos[pos][i])))
				else:
					self.table1.setItem(pos,i,GrisTableItem(self.table1,QTableItem.OnTyping,repr(self.__datos[pos][i]))) # 
			else:
				self.table1.setText(pos,i,repr(self.__datos[pos][i]))

	def __showtitulodat(self,pos=None):
		"""Actualiza los titulos de la tabla de datos segun las variables"""
		titulos=self.table1.horizontalHeader()
		if pos:
			titulos.setLabel(pos,self.__datos.getVar(pos).nombre)
		else:
			i=0
			for nombre in self.__datos.arrTit():
				titulos.setLabel(i,nombre)
				i=i+1

	
			

	def __combotableitem(self,indice=0):
		"""Devuelve un nuevo objeto tipo combotableitem"""
		lista=QStringList()
		from Driza import listas
		for tipo in listas.variables():
			lista.append(tipo)
		return QComboTableItem(self.table2,lista)
	
	def __buttontableitem(self):
		"""Devuelve un nuevo objeto mixto"""
		from Driza.widgets.widgetsqt import ButtonTableItem
		widget = ButtonTableItem(self.table2)
		return widget

	def __salirprograma(self):
		"""Funcion de salida del programa"""
		qApp.exit(0)

	def __nuevo(self):
		if self.__gestorfichero.fichero:
			#preguntar si quieres guardar
			rc = QMessageBox.information(self,'Atencion:', 'El fichero actual ha sido modificado, desea guardarlo?', 'Guardarlo','No guardarlo','Volver',0,1)
			if rc==0:
				self.__gestorfichero.guardar()
			elif rc==2:
				return
			self.__datos.borrartodo()
			self.__gestorfichero.fichero=None
			self.show()
		else:
			self.__datos.borrartodo()
			self.__gestorfichero.fichero=None
			self.show()


	
	def __guardar(self):
		"""Funcion de guardado del fichero. Si el fichero no es conocido se llama a guardarcomo"""
		if self.__gestorfichero.fichero:
			self.__gestorfichero.guardar()
		else:
			self.__guardarcomo()

	def __guardarcomo(self):
		"""Abre un diálogo pidiendo el nombre de archivo y guarda en dicho archivo"""
		filter=""
		for fmt in listas.extensiones_fichero():
			filter=filter+"%s files (*.%s);;" % (fmt,string.lower(fmt))
		fn = QFileDialog.getSaveFileName(QString.null,filter,self)
		filename = str(fn)
		if filename:
			from Driza.excepciones import FicheroExisteException
			try:
				self.__gestorfichero.guardar(filename)
			except FicheroExisteException,fichero:
				rc = QMessageBox.information(self,'Atencion:', 'El fichero'+fichero.fichero+' ya existe' , 'Sobreescribir','Otro nombre','Cancelar',0,1)
				if rc==0:
					self.__gestorfichero.guardar(filename,True)
				elif rc==1:
					self.__guardarcomo()
					


	def __insertardat(self,x,y,valor):
		"""Inserta un dato con la garantia de que ya existe el registro y la variable"""
		try:
			self.__datos[x][y]=self.__datos.getVar(y).nuevoitem(valor)
		except ValueError:
			rc = QMessageBox.warning(self,'Bienvenido', 'El dato no es válido')
	
	def __insertarregistro(self):
		"""Inserta un registro genérico, y ademas comprueba que no nos estamos acercando al final de la tabla"""
		self.__datos.appReg()
		if self.__nreg>(self.table1.numRows()-self.__datos.numReg()):
			self.table1.setNumRows(self.table1.numRows()+1)

	def __insertarvariable(self):
		"""Inserta una variable genérica, y ademas comprueba que no nos acercamos a ninguno de los limites de las tablas"""
		self.__datos.appVar()
		if self.__nvar>(self.table2.numRows()-self.__datos.numVar()):
			self.table2.setNumRows(self.table2.numRows()+1)
			self.table1.setNumCols(self.table1.numCols()+1)

	def __actualizardat(self,x,y):
		"""Actualiza los datos del objeto dato a partir de un cambio en la tabla de datos"""
		self.__portero.guardarestado()
		#from Driza import datos
		valor=self.table1.text(x,y).latin1()
		if y>=self.__datos.numVar():
			#Estamos trabajando sobre una variable inexistente
			for i in range(self.__datos.numVar(),y):
				self.__insertarvariable()
			self.__datos.appVar()
			self.__showtablavar() #actualizamos la tabla de variables
		if x>=self.__datos.numReg():
			#no existen registros intermedios
			for i in range (self.__datos.numReg(),x):  
				self.__insertarregistro()
				self.__showtabladat(i)
			self.__datos.appReg()
		#Actualizamos la fila que ha utilizado el usuario
		self.__insertardat(x,y,valor)
		self.__showtabladat()
		self.__showundoredo()

		
	def __actualizarvar(self,x,y):
		"""Funcion a que conecta con la introduccion de nuevos datos en la tabla de variables"""
		self.__portero.guardarestado()
		if x>=self.__datos.numVar(): #Es una nueva variable
			#Variables intermedias (creadas con los valores por defecto)
			for valorintermedio in range (self.__datos.numVar(),x):  
				self.__datos.appVar()
				self.__showtablavar(valorintermedio)
			self.__datos.appVar()
		#Variable modificada
		variable=self.__datos.getVar(x)
		if y==1:   # El usuario quiere cambiar el tipo
			from Driza import variables
			textoencuestion=self.table2.text(x,y).latin1() 
			if textoencuestion == "Entero":
				nuevavariable=variables.TEntero()
			elif textoencuestion == "Real":
				nuevavariable=variables.TReal()
			elif textoencuestion == "Booleano":
				nuevavariable=variables.TBooleano()
			elif textoencuestion == "Fecha":
				nuevavariable=variables.TFecha()
			nuevavariable.update(variable)
			self.__datos.setVar(x,nuevavariable)
			self.__showtabladat()
		elif y==2: # Estamos en el caso de los casos, mejor no hacer nada ... de momento
			print "No implementado"
		else: #Restos de campos (Texto)
			variable[y]=str(self.table2.text(x,y).latin1())
		self.__showtablavar()  
		#En todos los casos, actualizamos el titulo de la tabla de datos
		self.__showtitulodat()
		self.__showundoredo()


	def __removeselection(self,tabla):
		""" Esta funcion borra la seleccion de la tabla"""
		for seleccion in range(tabla.numSelections()):
			tabla.removeSelection(seleccion)

	def __deleteselection(self,tabla):
		""" Esta funcion borra la seleccion de la tabla,junto con los datos"""
		lista=[]
		for seleccion in range(tabla.numSelections()):
			for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
				lista.append(fila)
		lista.sort()
		lista.reverse()
		for fila in lista:
			del self.__datos[fila]
		tabla.removeSelection(seleccion)

	def __copiar(self):
		"""Esta funcion pega el contenido del clipboard en la tabla actual"""
		tablaactual=self.tabWidget2.currentPageIndex()
		if tablaactual == 0:
			tabla=self.table1
			cadena=""
			lista=[]
			for seleccion in range(tabla.numSelections()):
				for fila in range(tabla.selection(seleccion).topRow(),tabla.selection(seleccion).bottomRow()+1):
					lista.append(fila)
			lista.sort() #En la lista estan todas las filas que se nos piden y ordenadas
			for x in range(len(lista)):
				for y in range(self.__datos.numVar()):
					cadena+=repr(self.__datos[lista[x]][y])+":"
				cadena+=";"
			cb = QApplication.clipboard()
			cb.setData(QTextDrag(cadena))

	def __dcalculodescriptivo(self):
		"""Muestra el dialogo de calculo descriptivo"""
		self.__doperaciones.show("descriptivo")

	def __dcalculojicuadrado(self):
		"""Muestra el dialogo ji cuadrado"""
		self.__doperaciones.show("jicuadrado")

	def __dcalculoks(self):
		"""Muestra el dialogo ks"""
		self.__doperaciones.show("ks")

	def __dcalculocmedias(self):
		"""Muestra el dialogo de comparacion de medias"""
		self.dcalculo.show(3)

	def __dcalculoregresion(self):
		"""Muestra el dialogo de regresion"""
		self.__doperaciones.show("regresion")

	def __dgraficoshistograma(self):
		"""Muestra el la seccion de creación de histogramas"""
		self.__doperaciones.show("histograma")

	def __dgraficosdispersion(self):
		"""Muestra el la seccion de creación de histogramas"""
		self.__doperaciones.show("ddispersion")

	def __ddistribucionnormal(self):
		"""Muestra la distribucion normal"""
		self.__doperaciones.show("dnormal")
